@@ -12,10 +12,10 @@
  * man-page or http://www.ee.ryerson.ca:8080/~elf/xmotd.html
  * */
 
-/* $Id: main.c,v 1.14 1997/09/04 18:30:34 elf Exp $ */
+/* $Id: main.c,v 1.17 1999/11/23 22:55:14 elf Exp $ */
 
 /*
- * Copyright 1993-97  Luis Fernandes <elf@ee.ryerson.ca> 
+ * Copyright 1993-97, 1999  Luis Fernandes <elf@ee.ryerson.ca> 
  *
  * Permission to use, copy, hack, and distribute this software and its
  * documentation for any purpose and without fee is hereby granted,
@@ -67,6 +67,7 @@
 
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
+#include <X11/Xmu/Editres.h>
 
 #ifdef HAVE_XPM
 #include <X11/xpm.h>
@@ -144,6 +145,7 @@ static int alreadyForked;		/* flag to remember if we are already
 static XtResource resources[] = {
   { "always","Always",XtRInt, sizeof(int),
 	  offset(always),XtRString, "0"},
+
   { "popdown","Popdown",XtRInt, sizeof(int),
 	  offset(pto),XtRString, "0"},
   { "usedomains","UseDomains",XtRInt, sizeof(int),
@@ -152,6 +154,8 @@ static XtResource resources[] = {
 	  offset(showfilename),XtRString, "0"},
   { "paranoid","Paranoid",XtRInt, sizeof(int),
 	  offset(paranoid),XtRString, "0"},
+  { "tail","Tail",XtRInt, sizeof(int),
+	offset(tail),XtRString, "0"},
   { "bitmaplogo","BitmapLogo",XtRString, sizeof(String),
 	  offset(logo),XtRString, NULL},
   { "warnfile","Warnfile",XtRString, sizeof(String),
@@ -160,6 +164,8 @@ static XtResource resources[] = {
 	  offset(periodic),XtRString, "0"},
   { "stampfile","Stampfile",XtRString, sizeof(String),
 	  offset(stampfile),XtRString, TIMESTAMP},
+  { "atom","Atom",XtRString, sizeof(String),
+	  offset(atomname),XtRString, ATOM},
 #ifdef HAVE_HTML
   { "browser","Browser",XtRString, sizeof(String),
 	  offset(browser),XtRString, BROWSER},
@@ -171,16 +177,19 @@ static XrmOptionDescRec options[] = {
   { "-showfilename", "showfilename", XrmoptionNoArg,  "1"},
   { "-usedomains",   "usedomains",   XrmoptionNoArg,  "1"},
   { "-paranoid",     "paranoid",     XrmoptionNoArg,  "1"},
+  { "-tail",         "tail",         XrmoptionNoArg,  "1"},
   { "-popdown",      "popdown",      XrmoptionSepArg, (caddr_t) NULL},
   { "-bitmaplogo",   "bitmaplogo",   XrmoptionSepArg, (caddr_t) NULL},
   { "-warnfile",     "warnfile",     XrmoptionSepArg, (caddr_t) NULL},
   { "-wakeup",       "wakeup",       XrmoptionSepArg, (caddr_t) NULL},
   { "-stampfile",    "stampfile",    XrmoptionSepArg, (caddr_t) NULL},
+  { "-atom",         "atom",         XrmoptionSepArg, (caddr_t) NULL},
 #ifdef HAVE_HTML
   { "-browser",      "browser",      XrmoptionSepArg, (caddr_t) NULL},
 #endif
 };
 
+/* You can use shift + btn1 to quit xmotd (when run with -wakeup)*/
 static char shift1Trans[]="#override\n\
         Shift<Btn1Down>,Shift<Btn1Up>: reallyquit()";
 
@@ -188,6 +197,10 @@ static XtActionsRec xlations[]={
         {"reallyquit", (XtActionProc) reallyQuit},
 };
 
+/* when the text widget is mapped and we are in "tail" mode, scroll to
+   the end of the file*/
+static char tailTrans[]="#override\n\
+        <Expose>: end-of-file()";
 
 char *
 getTimeStampName()
@@ -477,9 +490,11 @@ main(argc, argv)
 int argc;
 char **argv;
 {
+  extern Boolean atomExists(String);
   Display *display;
   register int i, start=0;
   int numsg;
+
   
   if ((argc > 1) && !(strcmp(argv[1],"-help")))
 	{
@@ -559,12 +574,18 @@ char **argv;
 	  XtDestroyApplicationContext(app_con);		
 	  exit(0);
 	}  
-  
-  if(app_res.periodic)
+
+  if(app_res.periodic)			/*-wakeup or -timeout specified*/
 	{
-	  /* if -wakeup is specified we have to daemonize ourselves*/
-	  if(fork()) exit(0);
-	  alreadyForked=1;
+
+	  /*ensure no other copies of xmotd are running*/
+	  if(atomExists(app_res.atomname)){
+		XtDestroyApplicationContext(app_con);		
+		exit(0);
+	  }
+
+	  if(fork()) exit(0);		/*we have to daemonize ourselves*/
+	  alreadyForked=1;			/* make a note of it */
 
 	  if(!numsg)
 		{
@@ -577,6 +598,9 @@ char **argv;
   createWidgets(numsg);
   nextMessage((Widget)NULL, (caddr_t)NULL, (caddr_t)NULL);  
 
+  XtAddEventHandler(topLevel, (EventMask)0, True,
+					(XtEventHandler)_XEditResCheckMessages, 0);
+
   XtRealizeWidget(topLevel);  
   XtAppMainLoop(app_con);
 }
@@ -585,7 +609,7 @@ char **argv;
 createWidgets(int anymsg)
 {
   Widget form, paned, logo, mlabel, hline;
-  XtTranslations shift1TransTable;
+  XtTranslations shift1TransTable, tailTransTable;
   Pixel fg, bg;
   Arg args[8];
   int n;
@@ -600,7 +624,7 @@ createWidgets(int anymsg)
   form=XtVaCreateManagedWidget("form", formWidgetClass,topLevel, 
 							   XtNresizable, True,
 							   NULL);
-#endif
+#endif /* ifdef MOTIF */
 
   XtVaGetValues(form, 
 				XtNbackground, &bg,
@@ -680,7 +704,7 @@ createWidgets(int anymsg)
 				NULL);
   
   XtManageChild(text);
-#endif
+#endif /* ifdef HAVE_HTML & ifdef MOTIF */
 
 #else
 
@@ -726,9 +750,15 @@ createWidgets(int anymsg)
 								 XtNfromVert, info,
 								 NULL);
 
-#endif
-	
-#endif
+#endif /* ifdef HAVE_HTML */
+
+  if(app_res.tail)
+	{
+	  tailTransTable=XtParseTranslationTable(tailTrans);
+	  XtOverrideTranslations(text, tailTransTable);
+	}
+  
+#endif /* ifdef MOTIF */
 	
   if((anymsg>1))
 	{
