@@ -1,6 +1,6 @@
-/* $Id: usage.c,v 1.4 1997/06/23 17:41:06 elf Exp $ */
+/* $Id: usage.c,v 1.6 1999/11/23 22:59:46 elf Exp $ */
 /*
- * Copyright 1994-97 Luis Fernandes <elf@ee.ryerson.ca> 
+ * Copyright 1994-97, 1999 Luis Fernandes <elf@ee.ryerson.ca> 
  *
  * Permission to use, copy, hack, and distribute this software and its
  * documentation for any purpose and without fee is hereby granted,
@@ -61,6 +61,8 @@ char *str;
   fprintf(stderr, "   -usedomains       append domain-names to timestamp\n");
   fprintf(stderr, "   -wakeup #.#       every # hours, check motd(s) for changes\n");
   fprintf(stderr, "   -warnfile file    show the warning message in \"file\" before motd(s)\n");
+  fprintf(stderr, "   -atom name        register xmotd with \"name\"\n");
+  fprintf(stderr, "   -tail             scroll & display end of file\n");
 
   fprintf(stderr, "\n");
   exit(1);
